
var SAM={};
var modelInterface;
var MasterPropertyMap = {};
var ReverseMasterPropertyMap = {};
var hashArray;		// You need this here so the reference to the class will work below.

// Repositories
var queue_key_map;	// maps queue objectFullName to hasharray classkey
var fc_key_map;		// maps forwarding class objectFullName to hasharray classkey
var fc_fc_map;		// maps forwarding class objectFullName to the actual forwarding class name
var accInt_ing_key_map;	// maps access interface objectFullName to hasharray classkey (ingress)
var accInt_eg_key_map;	// maps access interface objectFullName to hasharray classkey (egress)
var accInt_storage;    // used for sorting access interfaces for later use
var accInt_by_policy;	// accessInterfaces grouped by policy (grouped by classkey)
var policy_key_map;	// maps policy objectFullName to hasharray classkey
var policy_def_fc;	// stores the default fc for each policy (indexed by classkey)


PV.adaptor.modelInterface = PV.ModelInterface();
PV.adaptor.modelInterface.retainIdMap = true;
modelInterface = PV.adaptor.modelInterface;
modelInterface.idMapFunction=function(anObject)
{
get_idmap_id(anObject)
};
modelInterface.onInsert=modelInterface.idMapFunction;
modelInterface.initializeForInventory();

PV.import('AppLogger.js')
PV.import('SAMDomainModel.js')
PV.import('SAMIF_selectJmsVersion.js')
PV.import('SAM_hash_array.js')
PV.import('SAMCommon.js')

PV.import('SAMIF_equipment_physical_port.js')
PV.import('SAMIF_equipment_shelf.js')
PV.import('SAMIF_equipment_hw_environment.js')
PV.import('SAMIF_service_access_interface.js')
PV.import('SAMIF_svt_sdp_binding.js')
PV.import('SAMIF_mpls_dynamic_lsp.js')
PV.import('SAMIF_mpls_interface.js')
PV.import('SAMIF_lag_interface.js')
PV.import('SAMIF_aingr_aengr_queue.js')
PV.import('SAMIF_aingr_aengr_forwarding_class.js')
PV.import('SAMIF_aingr_aengr_policy.js')

var fcQByType;
fcQByType = new hashArray();

var commit_count = 0;     // Used to do inline commits every 500 or so subelements

// propList is a list of the properties we want to update on the subelements
// when they change.

var propList = {"administrativeState":"prop",
		"equipmentState":"prop",
		"isLinkUp":"prop",
		"operationalState":"prop",
		"temperature":"prop"};


//*******************************************************************
//
// initializeSchemas
//
//*******************************************************************
PV.adaptor.initializeSchemas = function initializeSchemas() 
{
    this.schemas.add
    (
     PV.PvSAMDeviceSchema
     (
      "SAM",
             {
        	     jmsClasses: ["equipment.PhysicalPort", "equipment.Shelf", "equipment.HwEnvironment", "mpls.Interface", "mpls.Lsp", "mpls.DynamicLsp", "mpls.StaticLsp", "lag.Interface", "vll.L2AccessInterface", "vprn.L3AccessInterface", "ies.L3AccessInterface", "vpls.L2AccessInterface", "mirror.L2AccessInterface", "aengr.ForwardingClass", "aingr.ForwardingClass", "aengr.Queue", "aingr.Queue", "svt.MeshSdpBinding", "svt.MirrorSdpBinding", "svt.SpokeSdpBinding", "aingr.Policy", "aengr.Policy"],
 		     jmsProperties: {"newValue/long":"newvalue","newValue/int":"newvalue","newValue/string":"newvalue","oldValue/long":"oldvalue","oldValue/int":"oldvalue","oldValue/string":"oldvalue"},
		     jmsEvents: {
	"AttributeValueChange" : [[{xpath:XPathExp("//attribute")},[{xpath:XPathExp("attributeName")}, {xpath:XPathExp("newValue/long")}, {xpath:XPathExp("newValue/int")}, {xpath:XPathExp("newValue/string")}, {xpath:XPathExp("oldValue/long")}, {xpath:XPathExp("oldValue/int")}, {xpath:XPathExp("oldValue/string")}]]],
	"ObjectCreation" : [[{xpath:XPathExp("//objectCreationEvent")},[{xpath:XPathExp("*/*")}]]],
	"ObjectDeletion" : [[{xpath:XPathExp("//objectDeletionEvent")},[{xpath:XPathExp("objectFullName")}]]],
	"generic.ObjectDeletion" : [[{xpath:XPathExp("//objectDeletionEvent")},[{xpath:XPathExp("objectFullName")}]]],
	"KeepAliveEvent" : [],               
	"SystemInfoEvent" : [[{xpath:XPathExp("//stateChangeEvent")},[{xpath:XPathExp("*")}]]]
				},
  			     
                 onStartup: 
                 function() 
                 {
			 initialize_repositories();

                         return initializeDomainModel();
                 },
                 onJMSEvent:
                 function(event)
                 {
                         logP0Msg("onJMSEvent", "SAMIF", "entered");
//                         logP0Msg("onJMSEvent", "SAMIF", event.join("::"));
// make these P5 or something
                         handle_jms_msg(event);
                 },                 
                 onFullDump: 
                 function() 
                 {
              		 logP0Msg("onFullDump", "SAMIF", "**** Full Dump Started ****.");

                         logP0Msg("onFullDump", "SAMIF", "Clearing the repositories...");
			 fc_key_map.deleteFiles();
			 fc_fc_map.deleteFiles();
			 queue_key_map.deleteFiles();
			 accInt_ing_key_map.deleteFiles();
			 accInt_eg_key_map.deleteFiles();
			 accInt_storage.deleteFiles();
			 accInt_by_policy.deleteFiles();
	 		 policy_def_fc.deleteFiles();
	 		 policy_key_map.deleteFiles();

			 fc_key_map.close();
			 fc_fc_map.close();
			 queue_key_map.close();
			 accInt_ing_key_map.close();
			 accInt_eg_key_map.close();
			 accInt_storage.close();
			 accInt_by_policy.close();
	 		 policy_def_fc.close();
	 		 policy_key_map.close();

			 initialize_repositories();

                         processInventory(initializeSamObjects(), modelInterface);
//                         modelInterface.commitAll();
			 modelInterface.commit();
                         logP0Msg("onFullDump", "SAMIF", "****************************.");
                         logP0Msg("onFullDump", "SAMIF", "**** Full Dump Complete ****.");
                         logP0Msg("onFullDump", "SAMIF", "****************************.");
                 },
                 onOpen:
                 function()
                 {
                 		 logP4Msg("onOpen", "SAMIF", "Entered");                        
                 }
                 
             }
      )
     )
}


//*******************************************************************
//
// processInventory -- iterate through the list of objects and its
//                     properties, and then commit them.
// 
// Inputs:
//         theSamObjects   -- an initialised list of objects
//         modelInterface  -- the foreign domain model for SAM
//
// Output: nothing.
//
//*******************************************************************
function processInventory(theSamObjects, modelInterface)
{
    logP3Msg("processInventory", "SAMIF", "processInventory is starting.");
    





    theSamObjects.aengr_policy.enumerate(function(aSamObject){if (aSamObject!= undefined) process_aengr_policy(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "aengr_policy is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.aengr_policy.drop();
    logP4Msg("processInventory", "SAMIF", "aengr_policy has committed.");





    logP4Msg("processInventory", "SAMIF", "processing aingr_policy.");
    theSamObjects.aingr_policy.enumerate(function(aSamObject){if (aSamObject!= undefined) process_aingr_policy(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "finished processing aingr_policy.");
    definite_commit(modelInterface);;
    theSamObjects.aingr_policy.drop();
    logP4Msg("processInventory", "SAMIF", "aingr_policy has committed.");




    //...................................................................
    //
    //                       Q u e u e s
    //
    //...................................................................
    // We accumulate properties for the queues from the forwarding classes
    // so make sure to do those first.

    logP4Msg("processInventory", "SAMIF", "processing aingr_queue.");
    theSamObjects.aingr_queue.enumerate(function(aSamObject){if (aSamObject!= undefined) aingr_aengr_queue(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "aingr_queue is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.aingr_queue.drop();
    logP4Msg("processInventory", "SAMIF", "aingr_queue has committed.");


    logP4Msg("processInventory", "SAMIF", "processing aengr_queue.");
    theSamObjects.aengr_queue.enumerate(function(aSamObject){if (aSamObject!= undefined) aingr_aengr_queue(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "aengr_queue is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.aengr_queue.drop();
    logP4Msg("processInventory", "SAMIF", "aengr_queue has committed.");






    //...................................................................
    //
    //               F o r w a r d i n g   C l a s s e s
    //
    //...................................................................

    // There is a dependency because we accumulate data for setting up    
    // the Queue subelements.  We need to process the forwarding classes
    // then the access interfaces, then the queues.

    logP4Msg("processInventory", "SAMIF", "processing aingr_forwarding_class.");
    theSamObjects.aingr_forwarding_class.enumerate(function(aSamObject){if (aSamObject!= undefined) process_aingr_aengr_forwarding_class(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "finished processing aingr_forwarding_class.");
    definite_commit(modelInterface);;
    theSamObjects.aingr_forwarding_class.drop();
    logP4Msg("processInventory", "SAMIF", "aingr_forwarding_class has committed.");

    logP4Msg("processInventory", "SAMIF", "processing aengr_forwarding_class.");
    theSamObjects.aengr_forwarding_class.enumerate(function(aSamObject){if (aSamObject!= undefined) process_aingr_aengr_forwarding_class(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "finished processing aengr_forwarding_class.");
    definite_commit(modelInterface);;
    theSamObjects.aengr_forwarding_class.drop();
    logP4Msg("processInventory", "SAMIF", "aingr_forwarding_class has committed.");






    //...................................................................
    //
    //               A c c e s s I n t e r f a c e
    //
    //...................................................................

    // We need to process these after the forwarding classes and the
    // queues, because this is where we really create the queue
    // subelements, and we collect properties from the other two
    // SAM object classes.

    theSamObjects.vll_l2_access_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) process_access_interface(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "vll_l2_access_interface is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.vll_l2_access_interfaces.drop();
    logP4Msg("processInventory", "SAMIF", "vll_l2_access_interface has committed.");
    
    theSamObjects.vprn_l3_access_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) process_access_interface(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "vprn_l3_access_interface is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.vprn_l3_access_interfaces.drop();
    logP4Msg("processInventory", "SAMIF", "vprn_l3_access_interface has committed.");
    
    theSamObjects.vpls_l2_access_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) process_access_interface(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "vpls_l2_access_interface is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.vpls_l2_access_interfaces.drop();
    logP4Msg("processInventory", "SAMIF", "vpls_l2_access_interface has committed.");

    theSamObjects.mirror_l2_access_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) process_access_interface(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "mirror_l2_access_interface is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.mirror_l2_access_interfaces.drop();
    logP4Msg("processInventory", "SAMIF", "mirror_l2_access_interface has committed.");

    theSamObjects.ies_l3_access_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) process_access_interface(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "ies_l3_access_interface is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.ies_l3_access_interfaces.drop();
    logP4Msg("processInventory", "SAMIF", "ies_l3_access_interface has committed.");




    // The consolidation is done here, after the processing of all the service objects, since the
    // repositories all are used for storing service objects.  If it is done here then the
    // repositories are all set for when we start processing JMS, after the rest of the full dump has completed.

    logP0Msg("onFullDump", "SAMIF", "Consolidating the repositories...");
    fc_key_map.consolidate();
    fc_fc_map.consolidate();
    queue_key_map.consolidate();
    accInt_ing_key_map.consolidate();
    accInt_eg_key_map.consolidate();
    accInt_storage.consolidate();
    accInt_by_policy.consolidate();
    policy_def_fc.consolidate();
    policy_key_map.consolidate();


    //...................................................................
    //
    //               m p l s . I n t e r f a c e
    //
    //...................................................................
    theSamObjects.mpls_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) mpls_interface(aSamObject, modelInterface)});    
    logP4Msg("processInventory", "SAMIF", "mpls_interface is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.mpls_interfaces.drop();
    logP4Msg("processInventory", "SAMIF", "mpls_interface has committed.");

    //...................................................................
    //
    //               m p l s . L s p 
    //
    //...................................................................
    theSamObjects.mpls_dynamic_lsp.enumerate(function(aSamObject){if (aSamObject!= undefined) mpls_dynamic_lsp(aSamObject, modelInterface)});
    logP4Msg("processInventory", "SAMIF", "mpls_dynamic_lsp is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.mpls_dynamic_lsp.drop();
    logP4Msg("processInventory", "SAMIF", "mpls_dynamic_lsp has committed.");


    //...................................................................
    //
    //               s v t . S d p B i n d i n g
    //
    //...................................................................
    theSamObjects.mesh_sdp_binding.enumerate(function(aSamObject){if (aSamObject!= undefined) svt_sdp_binding(aSamObject, modelInterface)});    
    logP4Msg("processInventory", "SAMIF", "svt_mesh_sdp_binding is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.mesh_sdp_binding.drop();
    logP4Msg("processInventory", "SAMIF", "svt_mesh_sdp_binding has committed.");    

    theSamObjects.mirror_sdp_binding.enumerate(function(aSamObject){if (aSamObject!= undefined) svt_sdp_binding(aSamObject, modelInterface)});    
    logP4Msg("processInventory", "SAMIF", "svt_mirror_sdp_binding is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.mirror_sdp_binding.drop();
    logP4Msg("processInventory", "SAMIF", "svt_mirror_sdp_binding has committed.");    

    theSamObjects.spoke_sdp_binding.enumerate(function(aSamObject){if (aSamObject!= undefined) svt_sdp_binding(aSamObject, modelInterface)});    
    logP4Msg("processInventory", "SAMIF", "svt_spoke_sdp_binding is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.spoke_sdp_binding.drop();
    logP4Msg("processInventory", "SAMIF", "svt_spoke_sdp_binding has committed.");    


    //...................................................................
    //
    //               r t r . S e r v i c e I n t e r f a c e
    //
    //...................................................................
//    theSamObjects.service_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) rtr_service_interface(aSamObject, modelInterface)});            
//    logP4Msg("processInventory", "SAMIF", "rtr_service_interface is about to be committed.");
//    definite_commit(modelInterface);;
//    theSamObjects.service_interfaces.drop();
//    logP4Msg("processInventory", "SAMIF", "rtr_service_interface has committed.");
    
    //...................................................................
    //
    //               e q u i p m e n t . P h y s i c a l P o r t s
    //
    //...................................................................
    theSamObjects.physical_ports.enumerate(function(aSamObject){if (aSamObject!= undefined) equipment_physical_port(aSamObject, modelInterface)});                
    logP4Msg("processInventory", "SAMIF", "equipment_physical_port is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.physical_ports.drop();
    logP4Msg("processInventory", "SAMIF", "equipment_physical_port has committed.");

    //...................................................................
    //
    //               e q u i p m e n t . S h e l f
    //
    //...................................................................
    theSamObjects.shelfs.enumerate(function(aSamObject){if (aSamObject!= undefined) equipment_shelf(aSamObject, modelInterface)});    
    logP4Msg("processInventory", "SAMIF", "equipment_shelf is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.shelfs.drop();
    logP4Msg("processInventory", "SAMIF", "equipment_shelf has committed.");

    //...................................................................
    //
    //               l a g . I n t e r f a c e
    //
    //...................................................................
    logP4Msg("processInventory", "SAMIF", "before lag_interface");
    theSamObjects.lag_interfaces.enumerate(function(aSamObject){if (aSamObject!= undefined) lag_interface(aSamObject, modelInterface)});    
    logP4Msg("processInventory", "SAMIF", "lag_interface is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.lag_interfaces.drop();
    logP4Msg("processInventory", "SAMIF", "lag_interface has committed.");
    
    //...................................................................
    //
    //               e q u i p m e n t . H w E n v i r o n m e n t
    //
    //...................................................................
    theSamObjects.hw_environment.enumerate(function(aSamObject){if (aSamObject!= undefined) equipment_hw_environment(aSamObject, modelInterface)});    
    logP4Msg("processInventory", "SAMIF", "equipment_hw_environment is about to be committed.");
    definite_commit(modelInterface);;
    theSamObjects.hw_environment.drop();
    logP4Msg("processInventory", "SAMIF", "equipment_hw_environment has committed.");    


    logP3Msg("processInventory", "SAMIF", "processInventory is finished.");




} // processInventory


//*******************************************************************
//
//          i n i t i a l i z e S a m O b j e c t s
//
//*******************************************************************
function initializeSamObjects()
{
    var samObjects = {};

    logP3Msg("initializeSamObjects", "SAMIF", "entered" );


    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject aengr.ForwardingClass" );
	samObjects.aengr_forwarding_class = SAM.aengr.ForwardingClass.findToFile("aengr.ForwardingClass","", "");

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject aengr.Policy" );
	samObjects.aengr_policy = SAM.aengr.Policy.findToFile("aengr.Policy","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject aengr.Queue" );
	samObjects.aengr_queue = SAM.aengr.Queue.findToFile("aengr.Queue","", "");            


    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject aingr.ForwardingClass" );
	samObjects.aingr_forwarding_class = SAM.aingr.ForwardingClass.findToFile("aingr.ForwardingClass","", "");

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject aingr.Policy" );
	samObjects.aingr_policy = SAM.aingr.Policy.findToFile("aingr.Policy","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject aingr.Queue" );
	samObjects.aingr_queue = SAM.aingr.Queue.findToFile("aingr.Queue","", "");


    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject vprn.L3accessinterface" );
	samObjects.vprn_l3_access_interfaces = SAM.vprn.L3AccessInterface.findToFile("vprn.L3AccessInterface","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject ies.L3accessinterface" );
	samObjects.ies_l3_access_interfaces = SAM.ies.L3AccessInterface.findToFile("ies.L3AccessInterface","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject vpls.L2accessinterface" );
	samObjects.vpls_l2_access_interfaces = SAM.vpls.L2AccessInterface.findToFile("vpls.L2AccessInterface","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject vll.L2accessinterface" );
	samObjects.vll_l2_access_interfaces = SAM.vll.L2AccessInterface.findToFile("vll.L2AccessInterface","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject mirror.L2accessinterface" );
	samObjects.mirror_l2_access_interfaces = SAM.mirror.L2AccessInterface.findToFile("mirror.L2AccessInterface","", "");

//    logP3Msg("initializeSamObjects", "SAMIF", "complete" );
//    return samObjects;
//}

//function sam_placeholder() {

//	var samObjects = {};

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject svt.MeshSdpBinding" );
	samObjects.mesh_sdp_binding = SAM.svt.MeshSdpBinding.findToFile("svt.MeshSdpBinding","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject svt.MirrorSdpBinding" );
	samObjects.mirror_sdp_binding = SAM.svt.MirrorSdpBinding.findToFile("svt.MirrorSdpBinding","", "");            

    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject svt.SpokeSdpBinding" );
	samObjects.spoke_sdp_binding = SAM.svt.SpokeSdpBinding.findToFile("svt.SpokeSdpBinding","", "");            

//    logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject rtr.serviceinterface" );
//	samObjects.service_interfaces = SAM.generic.GenericObject.exportClassSummary("rtr.ServiceInterface","flatCSV","all", "");        

	//SAM.equipment.PhysicalPort.fieldMap = MasterPropertyMap;
	logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject equipment.physicalPort" );
	samObjects.physical_ports = SAM.equipment.PhysicalPort.findToFile("equipment.PhysicalPort","", "");            
    
	logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject equipment.shelf" );
	samObjects.shelfs = SAM.equipment.Shelf.findToFile("equipment.Shelf","",""); 

	logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject equipment.HwEnvironment" );
	samObjects.hw_environment = SAM.equipment.HwEnvironment.findToFile("equipment.HwEnvironment","","");            			
	logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject lag.Interface" );
	samObjects.lag_interfaces = SAM.lag.Interface.findToFile("lag.Interface","","");            		

	logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject mpls.Interface" );
	samObjects.mpls_interfaces = SAM.mpls.Interface.findToFile("mpls.Interface","", "");

	logP4Msg("initializeSamObjects", "SAMIF", "initialise SamObject mpls.DynamicLsp" );
	samObjects.mpls_dynamic_lsp = SAM.mpls.DynamicLsp.findToFile("mpls.DynamicLsp","", "");


    logP3Msg("initializeSamObjects", "SAMIF", "complete" );
    return samObjects;
}


//*******************************************************************
//
//             g e t _ i d m a p _ i d
//
//*******************************************************************
function get_idmap_id(sourceObject)
{
    logP4Msg("get_idmap_id", "SAMIF", sourceObject.samObjectFullName );
//	logStatus("sourceObject.alt_idmap", sourceObject.alt_idmap);

// Invariant length check

if (sourceObject.invariant != undefined) {
	logP4Msg("get_idmap_id", "SAMIF", "invariant length is " + sourceObject.invariant.length + " for " + sourceObject.invariant);
	}


	if (sourceObject.alt_idmap != undefined) {
		logP4Msg("get_idmap_id", "SAMIF", "Using alternate idmap function");
    		PV.adaptor.modelInterface.addIdMap(sourceObject.alt_idmap.value.toString().concat(",").concat(sourceObject.id.toString()).concat(",").concat(sourceObject.element.collectorNumber.toString()));
//    		PV.adaptor.modelInterface.addIdMap(sourceObject.alt_idmap.toString().concat(",").concat(sourceObject.id.toString()).concat(",").concat(sourceObject.element.collectorNumber.toString()));
		} else {
		if (sourceObject.samObjectFullName != undefined) {
		logP4Msg("get_idmap_id", "SAMIF", "Using regular idmap function");
    		PV.adaptor.modelInterface.addIdMap(sourceObject.samObjectFullName.value.toString().concat(",").concat(sourceObject.id.toString()).concat(",").concat(sourceObject.element.collectorNumber.toString()));
		}
		    else
		    	logP0Msg("get_idmap_id", "SAMIF", "objectFullName undefined");
	}
}


//*******************************************************************
//
//             j m s - e v e n t s
//
//*******************************************************************
function jms_newElement(className, objectName, elements)
{
	logP4Msg("jms_newElement", "SAMIF", className+" :: "+objectName);
	//var filter = '<equal name="objectFullName" value="'+objectName+'" />';
	//PV.Logger.logInfo("jms_newElement", "SAMIF", "querying SAM with filter ==>"+filter );
	//var elements = SAM.generic.GenericObject.classFind(className,filter);
	//for ( var i = 0; i < elements.length; i++ )
	for (var i in elements)
	{
        switch(className)
        {
	     	case "vll.L2AccessInterface":
		case "vprn.L3AccessInterface":
		case "ies.L3AccessInterface":
		case "vpls.L2AccessInterface":
		case "mirror.L2AccessInterface":
        		process_access_interface(elements[i], modelInterface);
        		break;
        	case "mpls.Interface":
        		mpls_interface(elements[i], modelInterface);
        		break;
        	case "mpls.DynamicLsp":
        		mpls_dynamic_lsp(elements[i], modelInterface);
        		break;
        	case "equipment.PhysicalPort":
        		equipment_physical_port(elements[i], modelInterface);
        		break;
        	case "equipment.Shelf":
        		equipment_shelf(elements[i], modelInterface);
        		break;
        	case "equipment.HwEnvironment":
        		equipment_hw_environment(elements[i], modelInterface);
        		break;
		case "aingr.ForwardingClass":
		case "aengr.ForwardingClass":
			process_aingr_aengr_forwarding_class(elements[i], modelInterface);
			break;
		case "aingr.Queue":
		case "aengr.Queue":
			aingr_aengr_queue(elements[i], modelInterface);
			break;
		case "aingr.Policy":
		case "aengr.Policy":
			process_aingr_policy(elements[i], modelInterface);
			break;
		case "svt.MeshSdpBinding":
		case "svt.MirrorSdpBinding":
		case "svt.SpokeSdpBinding":
        		svt_sdp_binding(elements[i], modelInterface);
			break;
		case "lag.Interface":
        		lag_interface(elements[i], modelInterface);
			break;
        	default:
        		logP4Msg("jms_newElement", "SAMIF", "Skipping unexpected className "+className);
        }    
    }
    definite_commit(modelInterface);;
}


function jms_delElement(className, objectName, delColl)
{
	var myId=0, myId2=0;
	var currentObject;
	var qmap;
	var delName;
	var i;

	logP4Msg("jms_delElement", "SAMIF", "entered");
	logP4Msg("jms_delElement", "SAMIF", objectName);
	for( i in delColl)
{

	logStatus("i", i);
	nilStatus("dellColl[i]", delColl[i]);
	nilStatus("dellColl[i].objectFullName", delColl[i].objectFullName);

	delName = delColl[i].objectFullName;
	logP4Msg("jms_delElement", "SAMIF", "delete "+className+": "+delName);

//	PV.Logger.logInfo("jms_delElement", "SAMIF", delName);	
//	if ((className == "aingr.Queue") || (classname == "aengr.Queue")) {
//		qmap = queue_map[delName];
//		if (qmap != undefined) {
//				myId = modelInterface.idForName(qmap.alt_idmap);
//			}
//		}
//	else 


	switch (className) {
		case "aingr.Queue":
		case "aengr.Queue":
			delete_SEs_by_queue_fullname(delName.toString());
			break;
		case "aingr.ForwardingClass":
		case "aengr.ForwardingClass":
			// can these be deleted?
			break;
        	case "vll.L2AccessInterface":
		case "vprn.L3AccessInterface":
		case "ies.L3AccessInterface":
		case "vpls.L2AccessInterface":
		case "mirror.L2AccessInterface":
			delete_access_interface(delName.toString());
			break;
		case "aingr.Policy":
		case "aengr.Policy":
			// Need something here?
			break;
		default:
			myId = modelInterface.idForName(delName.toString());

			logStatus("myId", myId);

			if(myId != 0)
			{  
				currentObject = modelInterface.Subelement();
			logStatus("currentObject", currentObject);
				currentObject.id = myId;
//				currentObject.name = delName;
				currentObject.state = false;
				currentObject.commit();
		logP6Msg("jms_delElement", "SAMIF", "deleted (turned off): "+delName );

			} else
			{
		logP4Msg("jms_delElement", "SAMIF", "Could not find Object!!!: "+delName );
//		PV.Logger.logInfo("jms_delElement", "SAMIF", "Could not find Object!!!: "+delName );
			}
		} // switch


	} // for loop 
	definite_commit(modelInterface);;

}

function jms_infoEvent(eventType, eventStateColl)
{
for (var i in eventStateColl)
{
	if(eventStateColl[i].state == "jmsMissedEvents")
	{ 
		logP4Msg("jms_infoEvent", "SAMIF", "MISSED_EVENT"+eventType );
//		PV.Logger.logInfo("jms_infoEvent", "SAMIF", "MISSED_EVENT"+eventType );
	}
}
}


// propName and propValue are arrays/collections.  propName has the names of the properties
// that are being changed, and propValue has the corresponding values they have changed to.


function jms_propChange(className, objectName, propColl)
{
	logP4Msg("jms_propChange", "SAMIF", "entered");	
	logStatus("className", className);
	logStatus("objectName", objectName);
	logP4Msg("jms_propChange", "SAMIF", "propChange "+className+": "+objectName);

        switch(className)
        {
        	case "vll.L2AccessInterface":
		case "vprn.L3AccessInterface":
		case "ies.L3AccessInterface":
		case "vpls.L2AccessInterface":
		case "mirror.L2AccessInterface":
        		propChange_service_access_interface(className, objectName, propColl);
        		break;
		case "aingr.ForwardingClass":
		case "aengr.ForwardingClass":
			propChange_aingr_aengr_forwarding_class(className, objectName, propColl);
			break;
		case "aingr.Queue":
		case "aengr.Queue":
			propChange_aingr_aengr_queue(className, objectName, propColl);
			break;
		case "aingr.Policy":
			propChange_aingr_policy(className, objectName, propColl);
			break;
		case "aengr.Policy":
			// we don't do anything with this one for propChange
			break;
        	case "mpls.Interface":
			jms_simple_propChange_with_mapping(className, objectName, propColl, reverse_mpls_interface_Name, mplsInterfaceMapFunctions);
			break;
        	case "mpls.DynamicLsp":
			jms_simple_propChange_with_mapping(className, objectName, propColl, reverse_mpls_dynamic_lsp_Name, lspMapFunctions);
			break;
        	case "equipment.PhysicalPort":
			jms_simple_propChange_with_mapping(className, objectName, propColl, reverse_equipment_physical_port_Name, physicalMapFunctions);
        		break;
        	case "equipment.Shelf":
			jms_simple_propChange_with_mapping(className, objectName, propColl, reverse_equipment_shelf_Name, shelfMapFunctions);
			break;
        	case "equipment.HwEnvironment":
			jms_simple_propChange_with_mapping(className, objectName, propColl, reverse_equipment_hwEnvironment);
			break;
		case "svt.MeshSdpBinding":
		case "svt.MirrorSdpBinding":
		case "svt.SpokeSdpBinding":
			jms_simple_propChange_with_mapping(className, objectName, propColl, reverse_svt_sdp_binding_Name, sdpMapFunctions);
			break;
		case "lag.Interface":
			jms_simple_propChange_with_mapping(className, objectName, propColl, reverse_lag_interface_Name, physicalMapFunctions);
			break;

        	default:
        		logP4Msg("jms_propChange", "SAMIF", "Skipping unexpected className "+className);
        }

	definite_commit(modelInterface);;

}




function jms_simple_propChange(className, objectName, propColl, propArray)
{
	var myId=0;
	var currentObject;
	var qmap;
	var changed;
	var i;
	var propName, propValue;

	logP5Msg("jms_simple_propChange", "SAMIF", "entered");	

	myId = modelInterface.idForName(objectName);

	logP5Msg("jms_propChange", "SAMIF", "object: "+objectName);	
	logP5Msg("jms_propChange", "SAMIF", "myId = "+myId);

	if(myId != 0)
	{ 	
		currentObject = modelInterface.Subelement();
		currentObject.id = myId; 
		changed = 0;

		nilStatus("jms_propChange currentObject", currentObject);

//		for (var i in propColl) {
//		 	logP4Msg("jms_propChange", "SAMIF", "i="+i);
//			logStatus("jms_propChange propList", propColl[i].attributeName);
//			logStatus("jms_propChange propValue", propColl[i].newvalue);
//			}


		for( i in propColl)
		{    propName = propColl[i].attributeName;
		     propValue = propColl[i].newvalue;

			logP5Msg("jms_propChange", "SAMIF", "propName: "+propName);
			logP5Msg("jms_propChange", "SAMIF", "propValue: "+propValue);
			logStatus("jms_propChange propArray", propArray[propName]);
//			if (isDef(propList[propName])) { // a property we care about

			if (isDef(propArray[propName])) { // a property we care about
				changed=1;
			        logP5Msg("jms_propChange", "SAMIF", objectName+'==>['+propName+']::'+propValue);	
//				logP5Msg("jms_propChange", "SAMIF", MasterPropertyMap[propName]);
//				currentObject.addProperty(MasterPropertyMap[propName].toString(), propValue);
				logP5Msg("jms_propChange", "SAMIF", propArray[propName]);
				currentObject.addProperty(propArray[propName].toString(), propValue);
				}
			else {
				logP5Msg("jms_propChange", "SAMIF", propName+": skipping");	
				}
			}


		logStatus("jms_propChange changed", changed);
		if (changed == 1) {
			logP5Msg("jms_propChange", "SAMIF", "About to commit subelement");
	                currentObject.commit();
			logP5Msg("jms_propChange", "SAMIF", "Committed subelement");
		}

		definite_commit(modelInterface);;
		logP4Msg("jms_propChange", "SAMIF", "Committed model interface");

		}else
		{ //couldn't find object in idmap
			logP4Msg("jms_propChange", "SAMIF",  "Could not find Object!!!: "+objectName );
		}	

}



function jms_simple_propChange_with_mapping(className, objectName, propColl, propArray, mapArray)
{
	var myId=0;
	var currentObject;
	var qmap;
	var changed;
	var i;
	var propName, propValue, mapValue;

	logP5Msg("jms_simple_propChange", "SAMIF", "entered");	

	myId = modelInterface.idForName(objectName);

	logP5Msg("jms_propChange", "SAMIF", "object: "+objectName);	
	logP5Msg("jms_propChange", "SAMIF", "myId = "+myId);

	if(myId != 0)
	{ 	
		currentObject = modelInterface.Subelement();
		currentObject.id = myId; 
		changed = 0;

		nilStatus("jms_propChange currentObject", currentObject);

//		for (var i in propColl) {
//		 	logP4Msg("jms_propChange", "SAMIF", "i="+i);
//			logStatus("jms_propChange propList", propColl[i].attributeName);
//			logStatus("jms_propChange propValue", propColl[i].newvalue);
//			}


		for( i in propColl)
		{    propName = propColl[i].attributeName;
		     propValue = propColl[i].newvalue;

			logP4Msg("jms_propChange", "SAMIF", "propName: "+propName);
			logP4Msg("jms_propChange", "SAMIF", "propValue: "+propValue);
			logStatus("jms_propChange propArray", propArray[propName]);
//			if (isDef(propList[propName])) { // a property we care about

			if (isDef(mapArray) && isDef(mapArray[propName])) {
					mapValue = mapArray[propName](objectName, propName, propValue);
					changed=1;
			        	logP5Msg("jms_propChange", "SAMIF", objectName+'==>['+propName+']::'+propValue);	
			        	logP5Msg("jms_propChange", "SAMIF", propValue+" ==> "+mapValue);
					logP5Msg("jms_propChange", "SAMIF", propArray[propName]);
					currentObject.addProperty(propArray[propName].toString(), mapValue);
				}
											
			else if (isDef(propArray[propName])) { // a property we care about
				changed=1;
			        logP5Msg("jms_propChange", "SAMIF", objectName+'==>['+propName+']::'+propValue);	
//				logP5Msg("jms_propChange", "SAMIF", MasterPropertyMap[propName]);
//				currentObject.addProperty(MasterPropertyMap[propName].toString(), propValue);
				logP5Msg("jms_propChange", "SAMIF", propArray[propName]);
				currentObject.addProperty(propArray[propName].toString(), propValue);
				}
			else {
				logP5Msg("jms_propChange", "SAMIF", propName+": skipping");	
				}
			}


		logStatus("jms_propChange changed", changed);
		if (changed == 1) {
			logP5Msg("jms_propChange", "SAMIF", "About to commit subelement");
	                currentObject.commit();
			logP5Msg("jms_propChange", "SAMIF", "Committed subelement");
		}

		definite_commit(modelInterface);;
		logP4Msg("jms_propChange", "SAMIF", "Committed model interface");

		}else
		{ //couldn't find object in idmap
			logP4Msg("jms_propChange", "SAMIF",  "Could not find Object!!!: "+objectName );
		}	

}


function possible_inline_commit(modelInterface)
{

	commit_count = commit_count+1;
	if (commit_count >=500) {
		logP4Msg("possible_inline_commit", "SAMIF",  "Doing inline commit after "+commit_count+" sublements.");
		modelInterface.inlineCommit();
		commit_count = 0;
		}
}


function definite_commit(modelInterface)
{
	logP6Msg("definite_commit", "SAMIF",  "Committing model interface.");
	modelInterface.commit();
	commit_count = 0;
}


function initialize_repositories()
{

	 logP6Msg("initialize_repositories", "SAMIF", "Initializing fc_key_map repository");
	 fc_key_map = PV.Repository('fc_key_map');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing fc_fc_map repository");
	 fc_fc_map = PV.Repository('fc_fc_map');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing queue_key_map repository");
	 queue_key_map = PV.Repository('queue_key_map');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing accInt_ing_key_map repository");
	 accInt_ing_key_map = PV.Repository('accInt_ing_key_map');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing accInt_eg_key_map repository");
	 accInt_eg_key_map = PV.Repository('accInt_eg_key_map');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing accInt_storage repository");
	 accInt_storage = PV.Repository('accInt_storage');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing accInt_by_policy repository");
	 accInt_by_policy = PV.Repository('accInt_by_policy');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing policy_def_fc repository");
	 policy_def_fc = PV.Repository('policy_def_fc');
	 logP6Msg("initialize_repositories", "SAMIF", "Initializing policy_key_map repository");
	 policy_key_map = PV.Repository('policy_key_map');

}


function array_test() {

var i, j;

for (i in MasterPropertyMap) {
//	 logP6Msg("array_test", "SAMIF", "MasterPropertyMap index: "+i);
	 logP6Msg("array_test", "SAMIF", "MasterPropertyMap["+i+"] = "+MasterPropertyMap[i]);
	}


for (i in ReverseMasterPropertyMap) {
//	 logP6Msg("array_test", "SAMIF", "ReverseMasterPropertyMap index: "+i);
	 logP6Msg("array_test", "SAMIF", "ReverseMasterPropertyMap["+i+"] = "+ReverseMasterPropertyMap[i]);
	}


for (i in svc_access_interface_Name) {
//	 logP6Msg("array_test", "SAMIF", "svc_access_interface_Name index: "+i);
	 logP6Msg("array_test", "SAMIF", "svc_access_interface_Name["+i+"] = "+svc_access_interface_Name[i]);
	}

for (i in reverse_svc_access_interface_Name) {
//	 logP6Msg("array_test", "SAMIF", "reverse_svc_access_interface_Name index: "+i);
	 logP6Msg("array_test", "SAMIF", "reverse_svc_access_interface_Name["+i+"] = "+reverse_svc_access_interface_Name[i]);
	}


}

	
